; TIATracker music player (MADS format)
; Copyright 2016 Andre "Kylearan" Wichmann
; Website: https://bitbucket.org/kylearan/tiatracker
; Email: andre.wichmann@gmx.de
;
; Licensed under the Apache License, Version 2.0 (the "License");
; you may not use this file except in compliance with the License.
; You may obtain a copy of the License at
;
;   http://www.apache.org/licenses/LICENSE-2.0
;
; Unless required by applicable law or agreed to in writing, software
; distributed under the License is distributed on an "AS IS" BASIS,
; WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
; See the License for the specific language governing permissions and
; limitations under the License.

; Song author: BeWu
; Song name:   Little 50
; Code author: JAC!

; @com.wudsn.ide.asm.hardware=ATARI2600

;	Global cartridge locations
.enum mmu	;MMU Bank switching locations
	bank1 = $1ff8	;Any access activates bank 1
	bank2 = $1ff9	;Any access activates bank 2
.ende

	icl "VCS.asm"
	
; TV format switches
PAL	     = 1
NTSC	    = 0

	.if PAL
TIM_VBLANK      = 43
TIM_OVERSCAN    = 36
TIM_KERNEL      = 19
	.else
TIM_VBLANK      = 45
TIM_OVERSCAN    = 38
TIM_KERNEL      = 15
	.endif


; =====================================================================
; Variables
; =====================================================================

	opt h-f-
	org $80

	icl "Song_variables.asm"

startup_delay	.ds 1
player_time_max	 .ds 1
color		.ds 1
audf0_ram	.ds 2
audv0_ram	.ds 2
lda_tt_ptr_y	.ds .len bank1.lda_tt_ptr_y_rom

; =====================================================================
; Start of code
; =====================================================================

	opt f-
	org $d000
	opt f+

	.proc bank1 

start	bit mmu.bank1
 	jmp skip

	.byte 10,13,0
	.byte 'Little 15 by BeWu! for SillyVenture 2k22 Summer Edition',10,13,0
	.byte 'Info screen by JAC! on 2022-08-13',10,13,0

	.proc lda_tt_ptr_y_rom	;Mut not change V-flag, must change Z-flag
	php
	bit mmu.bank1
	lda tt_ptr+1
	cmp #>bank2
	bcc still_bank1
	bit mmu.bank2
still_bank1
	lda (tt_ptr),y
	bit mmu.bank1
	plp
	cmp #0
	rts
	.endp

skip
	.proc init
	cld
	ldx #0
	txa
clearLoop:
	dex
	txs
	pha
	bne clearLoop

	ldx #.len lda_tt_ptr_y_rom
copy_loop
	mva lda_tt_ptr_y_rom,x lda_tt_ptr_y,x
	dex
	bpl copy_loop

	icl "Song_init.asm"
	mva #150 startup_delay

	.endp
      
; =====================================================================
; MAIN LOOP
; =====================================================================

	.proc main_loop

	lda swchb		;RESET (F2) pressed?
	and #1
	jeq start

	.proc overscan       

	sta WSYNC
	lda #2
	sta VBLANK
	lda #TIM_OVERSCAN
	sta TIM64T

	; Do overscan stuff

:10	nop
	sta resp0
	sta resp1
	mva #70 hmp1
	sta wsync
	sta hmove
	sta wsync
	sta hmove
	lda #6
	sta nusiz0
	sta nusiz1

wait_for_intim
	lda INTIM
	bne wait_for_intim
	.endp

; ---------------------------------------------------------------------
; VBlank
; ---------------------------------------------------------------------

	.proc vblank

	lda #%1110
vsyncLoop:
	sta WSYNC
	sta VSYNC
	lsr
	bne vsyncLoop
	lda #2
	sta VBLANK
	lda #TIM_VBLANK
	sta TIM64T

	lda startup_delay
	beq play
	dec startup_delay
	jne dont_play
	; Do VBlank stuff
play	icl "Song_player.asm"
dont_play
	; Measure player worst case timing
	lda #TIM_VBLANK
	sec
	sbc INTIM
	cmp player_time_max
	bcc noNewMax
	sta player_time_max
noNewMax:

waitForVBlank:
	lda INTIM
	bne waitForVBlank
	sta WSYNC
	sta VBLANK
	.endp

; ---------------------------------------------------------------------
; Kernel
; ---------------------------------------------------------------------

	.proc kernel
	lda #TIM_KERNEL
	sta T1024T

	; Do kernel stuff
	ldx #0
center	stx wsync
	inx
	cpx #8
	bne center

	lda AUDF0_RAM+0
	and #$f0
	sta color
	lda AUDV0_RAM+0
	jsr vu_bar

	jsr playfield

	lda AUDF0_RAM+1
	asl
	asl
	asl
	asl
	and #$f0
	sta color
	lda AUDV0_RAM+1
	jsr vu_bar

wait_for_intim
	lda INTIM
	bne wait_for_intim
	.endp

	jmp main_loop
	.endp
; =====================================================================

	.proc playfield
;	jmp start.title.playfield

	ldx #0
upper	stx wsync
	stx colubk
	inx
	inx
	cpx #16
	bne upper

	ldx #0
effect	stx wsync
	lda colors,x
	sta colup0
	lda gfx.first+1,x
	ldy gfx.first,x
	sta grp0
	sty grp1
:4	nop
	lda gfx.second+1,x
	ldy gfx.second,x
	sta grp0
	sty grp1
	lda gfx.third+1,x
	ldy gfx.third,x
	sta grp0
	sty grp1
	inx
;	cpx #192-32*2
	cpx #127
	bne effect

	ldx #15
lower	stx wsync
	stx colubk
	dex
	dex
	bpl lower
	rts
	.endp

; =====================================================================

	.proc vu_bar
	and #15
	tay
	ldx #15
bar_up	lda lumas,y
	seq
	ora color
	sta wsync
	sta colubk
	iny
	dex
	bpl bar_up
	ldx #15
bar_down
	dey
	lda lumas,y
	seq
	ora color
	sta wsync
	sta colubk
	dex
	bpl bar_down
	rts
	.endp

; =====================================================================
; Data
; =====================================================================


picture_height = $80
picture
	
	.align $100

	.local gfx
first	ins "gfx/Song.pic",+picture_height*0,picture_height
second	ins "gfx/Song.pic",+picture_height*1,picture_height
third	ins "gfx/Song.pic",+picture_height*2,picture_height
	.endl
	
	m_info gfx

	.local colors
	red = $60
	olive = $40
	yellow = $20
	violet = $a0
	blue_violet = $c0
	blue_seven = $b0
	blue = $d0
	blue_green = $90
	green = $50
	gold = $20

	.macro m_col
;	.byte :1+$00
;	.byte :1+$00
	.byte :1+$02
	.byte :1+$04
	.byte :1+$06
	.byte :1+$08
	.byte :1+$0a
	.byte :1+$0c
	.byte :1+$0e
	.byte :1+$0c
	.byte :1+$0a

	.byte $00
	.endm

:4	.byte $00

	m_col red
	m_col yellow
	m_col olive	
	m_col green
	m_col blue_green
	m_col blue
	m_col blue_violet
	m_col violet
	m_col red
	m_col yellow
	m_col olive	
	m_col green
;	m_col blue_green
;	m_col blue
;	m_col blue_violet
;	m_col violet

	.endl
	
	m_info colors

	.local lumas
	.byte $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
	.byte $00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0a,$0b,$0c,$0d,$0e,$0f
	.endl

	icl "Song_trackdata.asm"
	.use bank2
	icl "Song_patterns_addresses.asm"
	icl "Song_patterns_1.asm"

	.echo "Bank 1 ROM left at: ",*, " has ", (start+$ff9 - *), " bytes"

	org start+$ff8

	org start+$ffc
	.word start
	.word start
	
	.endp			;End of bank1
 
; =====================================================================
; Vectors
; =====================================================================
 
 	opt f-
 	org $f000
 	opt f+
 
 	.proc bank2
 
start 	bit mmu.bank1
	jmp *		 	;This will continue on bank1

	icl "Song_patterns_2.asm"
	
; =====================================================================

	.echo "Bank 2 ROM left at: ",*, " has ", (start+$ff9 - *), " bytes"

	org start+$ff8

	org start+$ffc
	.word start
	.word start
	
	.endp			;End of bank2
;      
