; TIATracker music player  (MADS format)
; Copyright 2016 Andre "Kylearan" Wichmann
; Website: https://bitbucket.org/kylearan/tiatracker
; Email: andre.wichmann@gmx.de
;
; Licensed under the Apache License, Version 2.0 (the "License");
; you may not use this file except in compliance with the License.
; You may obtain a copy of the License at
;
;   http://www.apache.org/licenses/LICENSE-2.0
;
; Unless required by applicable law or agreed to in writing, software
; distributed under the License is distributed on an "AS IS" BASIS,
; WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
; See the License for the specific language governing permissions and
; limitations under the License.

; Song author: BeWu/Mystic Bytes
; Song name: VCS: Very Casual Song

; @com.wudsn.ide.asm.hardware=ATARI2600

; =====================================================================
; TIATracker melodic and percussion instruments, patterns and sequencer
; data.
; =====================================================================
tt_TrackDataStart:

; =====================================================================
; Melodic instrument definitions (up to 7). tt_envelope_index_c0/1 hold
; the index values into these tables for the current instruments played
; in channel 0 and 1.
; 
; Each instrument is defined by:
; - tt_InsCtrlTable: the AUDC value
; - tt_InsADIndexes: the index of the start of the ADSR envelope as
;       defined in tt_InsFreqVolTable
; - tt_InsSustainIndexes: the index of the start of the Sustain phase
;       of the envelope
; - tt_InsReleaseIndexes: the index of the start of the Release phase
; - tt_InsFreqVolTable: The AUDF frequency and AUDV volume values of
;       the envelope
; =====================================================================

; Instrument master CTRL values
tt_InsCtrlTable:
        .byte $04, $0c, $0c, $04, $0c, $06, $01


; Instrument Attack/Decay start indexes into ADSR tables.
tt_InsADIndexes:
        .byte $00, $00, $05, $09, $09, $14, $18


; Instrument Sustain start indexes into ADSR tables
tt_InsSustainIndexes:
        .byte $00, $00, $05, $10, $10, $14, $1c


; Instrument Release start indexes into ADSR tables
; Caution: Values are stored with an implicit -1 modifier! To get the
; real index, add 1.
tt_InsReleaseIndexes:
        .byte $01, $01, $06, $11, $11, $15, $1d


; AUDVx and AUDFx ADSR envelope values.
; Each byte encodes the frequency and volume:
; - Bits 7..4: Freqency modifier for the current note ([-8..7]),
;       8 means no change. Bit 7 is the sign bit.
; - Bits 3..0: Volume
; Between sustain and release is one byte that is not used and
; can be any value.
; The end of the release phase is encoded by a 0.
tt_InsFreqVolTable:
; 0+1: BeWu_sqr_hi
        .byte $86, $00, $86, $87, $00
; 2: BeWu_sqr_lo
        .byte $83, $00, $84, $00
; 3+4: BeWu_lead
        .byte $8d, $8d, $8b, $8a, $89, $88, $86, $84
        .byte $00, $83, $00
; 5: BeWu_bass_deep
        .byte $87, $00, $88, $00
; 6: BeWu_bass_sharp
        .byte $86, $86, $85, $85, $84, $00, $80, $00



; =====================================================================
; Percussion instrument definitions (up to 15)
;
; Each percussion instrument is defined by:
; - tt_PercIndexes: The index of the first percussion frame as defined
;       in tt_PercFreqTable and tt_PercCtrlVolTable
; - tt_PercFreqTable: The AUDF frequency value
; - tt_PercCtrlVolTable: The AUDV volume and AUDC values
; =====================================================================

; Indexes into percussion definitions signifying the first frame for
; each percussion in tt_PercFreqTable.
; Caution: Values are stored with an implicit +1 modifier! To get the
; real index, subtract 1.
tt_PercIndexes:
        .byte $01, $18, $21, $2c, $37, $42, $4d, $58
        .byte $63, $6e, $79, $84, $86, $8a, $95


; The AUDF frequency values for the percussion instruments.
; If the second to last value is negative (>=128), it means it's an
; "overlay" percussion, i.e. the player fetches the next instrument note
; immediately and starts it in the sustain phase next frame. (Needs
; TT_USE_OVERLAY)
tt_PercFreqTable:
; 0: BeWu_crash
        .byte $00, $01, $02, $02, $03, $03, $03, $03
        .byte $04, $04, $04, $04, $04, $05, $05, $05
        .byte $06, $06, $06, $07, $08, $0a, $00
; 1: BeWu_snare
        .byte $03, $1a, $05, $02, $03, $05, $06, $87
        .byte $00
; 2: BeWu_arp1
        .byte $00, $12, $0e, $0a, $12, $0e, $0a, $12
        .byte $0e, $0a, $00
; 3: BeWu_arp2
        .byte $00, $12, $0e, $12, $0e, $12, $0e, $12
        .byte $0e, $12, $00
; 4: BeWu_arp4
        .byte $00, $13, $10, $09, $13, $10, $09, $13
        .byte $10, $09, $00
; 5: BeWu_arp5
        .byte $00, $15, $10, $0c, $15, $10, $0c, $15
        .byte $10, $0c, $00
; 6: BeWu_arp6
        .byte $00, $15, $12, $0d, $15, $12, $0d, $15
        .byte $12, $0d, $00
; 7: BeWu_arp7
        .byte $00, $12, $0e, $18, $12, $0e, $18, $12
        .byte $0e, $18, $00
; 8: BeWu_arp8
        .byte $00, $12, $0e, $0b, $12, $0e, $0b, $12
        .byte $0e, $0b, $00
; 9: BeWu_arp9
        .byte $00, $0e, $0c, $09, $0e, $0c, $09, $0e
        .byte $0c, $09, $00
; 10: BeWu_arp10
        .byte $00, $0e, $0c, $12, $0e, $0c, $12, $0e
        .byte $0c, $12, $00
; 11: Hat
        .byte $80, $00
; 12: BeWu_bdrum
        .byte $05, $09, $8c, $00
; 13: BeWu_arp3
        .byte $00, $12, $0d, $0a, $12, $0d, $0a, $12
        .byte $0d, $0a, $00
; 14: BeWu_arp11
        .byte $00, $0e, $0b, $09, $0e, $0b, $09, $0e
        .byte $0b, $09, $00


; The AUDCx and AUDVx volume values for the percussion instruments.
; - Bits 7..4: AUDC value
; - Bits 3..0: AUDV value
; 0 means end of percussion data.
tt_PercCtrlVolTable:
; 0: BeWu_crash
        .byte $86, $86, $85, $85, $85, $85, $84, $84
        .byte $84, $84, $84, $84, $83, $83, $83, $83
        .byte $82, $82, $82, $82, $81, $80, $00
; 1: BeWu_snare
        .byte $89, $ca, $6a, $89, $87, $84, $81, $80
        .byte $00
; 2: BeWu_arp1
        .byte $83, $c9, $c9, $c8, $c8, $c7, $c6, $c5
        .byte $c3, $c1, $00
; 3: BeWu_arp2
        .byte $83, $c9, $c9, $c8, $c8, $c7, $c6, $c5
        .byte $c3, $c1, $00
; 4: BeWu_arp4
        .byte $83, $c9, $c9, $c8, $c8, $c7, $c6, $c5
        .byte $c3, $c1, $00
; 5: BeWu_arp5
        .byte $83, $c9, $c9, $c8, $c8, $c7, $c6, $c5
        .byte $c3, $c1, $00
; 6: BeWu_arp6
        .byte $83, $c9, $c9, $c8, $c8, $c7, $c6, $c5
        .byte $c3, $c1, $00
; 7: BeWu_arp7
        .byte $83, $c9, $c9, $c8, $c8, $c7, $c6, $c5
        .byte $c3, $c1, $00
; 8: BeWu_arp8
        .byte $83, $c9, $c9, $c8, $c8, $c7, $c6, $c5
        .byte $c3, $c1, $00
; 9: BeWu_arp9
        .byte $83, $c9, $c9, $c8, $c8, $c7, $c6, $c5
        .byte $c3, $c1, $00
; 10: BeWu_arp10
        .byte $83, $c9, $c9, $c8, $c8, $c7, $c6, $c5
        .byte $c3, $c1, $00
; 11: Hat
        .byte $84, $00
; 12: BeWu_bdrum
        .byte $6f, $6d, $69, $00
; 13: BeWu_arp3
        .byte $83, $c9, $c9, $c8, $c8, $c7, $c6, $c5
        .byte $c3, $c1, $00
; 14: BeWu_arp11
        .byte $83, $c9, $c9, $c8, $c8, $c7, $c6, $c5
        .byte $c3, $c1, $00


        
; =====================================================================
; Track definition
; The track is defined by:
; - tt_PatternX (X=0, 1, ...): Pattern definitions
; - tt_PatternPtrLo/Hi: Pointers to the tt_PatternX tables, serving
;       as index values
; - tt_SequenceTable: The order in which the patterns should be played,
;       i.e. indexes into tt_PatternPtrLo/Hi. Contains the sequences
;       for all channels and sub-tracks. The variables
;       tt_cur_pat_index_c0/1 hold an index into tt_SequenceTable for
;       each channel.
;
; So tt_SequenceTable holds indexes into tt_PatternPtrLo/Hi, which
; in turn point to pattern definitions (tt_PatternX) in which the notes
; to play are specified.
; =====================================================================

; ---------------------------------------------------------------------
; Pattern definitions, one table per pattern. tt_cur_note_index_c0/1
; hold the index values into these tables for the current pattern
; played in channel 0 and 1.
;
; A pattern is a sequence of notes (one byte per note) ending with a 0.
; A note can be either:
; - Pause: Put melodic instrument into release. Must only follow a
;       melodic instrument.
; - Hold: Continue to play last note (or silence). Default "empty" note.
; - Slide (needs TT_USE_SLIDE): Adjust frequency of last melodic note
;       by -7..+7 and keep playing it
; - Play new note with melodic instrument
; - Play new note with percussion instrument
; - End of pattern
;
; A note is defined by:
; - Bits 7..5: 1-7 means play melodic instrument 1-7 with a new note
;       and frequency in bits 4..0. If bits 7..5 are 0, bits 4..0 are
;       defined as:
;       - 0: End of pattern
;       - [1..15]: Slide -7..+7 (needs TT_USE_SLIDE)
;       - 8: Hold
;       - 16: Pause
;       - [17..31]: Play percussion instrument 1..15
;
; The tracker must ensure that a pause only follows a melodic
; instrument or a hold/slide.
; ---------------------------------------------------------------------
TT_FREQ_MASK    = %00011111
TT_INS_HOLD     = 8
TT_INS_PAUSE    = 16
TT_FIRST_PERC   = 17



; Individual pattern speeds (needs TT_GLOBAL_SPEED = 0).
; Each byte encodes the speed of one pattern in the order
; of the tt_PatternPtr tables below.
; If TT_USE_FUNKTEMPO is 1, then the low nibble encodes
; the even speed and the high nibble the odd speed.

    .if TT_GLOBAL_SPEED = 0
tt_PatternSpeeds:
        .byte $02, $02, $02, $02, $02, $02, $02, $02
        .byte $02, $02, $02, $02, $02, $02, $02, $02
        .byte $02, $02, $02, $02, $02, $02, $02, $02
        .byte $02, $02, $02, $02, $02, $02, $02, $02
        .byte $02, $02, $00, $02, $0e, $02, $02, $02
        .byte $02, $02, $02, $02, $02, $02, $02, $02
        .byte $02, $02, $02, $02, $02, $02, $02, $02
        .byte $02, $02, $02, $02, $02, $02, $02, $02
        .byte $02, $02, $02, $02, $02, $02

    .endif

; ---------------------------------------------------------------------
; Pattern sequence table. Each byte is an index into the
; tt_PatternPtrLo/Hi tables where the pointers to the pattern
; definitions can be found. When a pattern has been played completely,
; the next byte from this table is used to get the address of the next
; pattern to play. tt_cur_pat_index_c0/1 hold the current index values
; into this table for channels 0 and 1.
; If TT_USE_GOTO is used, a value >=128 denotes a goto to the pattern
; number encoded in bits 6..0 (i.e. value AND %01111111).
; ---------------------------------------------------------------------
tt_SequenceTable:
        ; ---------- Channel 0 ----------
        .byte $00, $01, $00, $02, $03, $04, $05, $06
        .byte $07, $04, $08, $09, $00, $01, $00, $01
        .byte $0a, $0b, $0c, $0d, $0e, $0f, $10, $11
        .byte $0e, $0f, $10, $12, $13, $14, $15, $16
        .byte $15, $14, $15, $17, $18, $19, $1a, $1b
        .byte $1c, $1d, $1e, $1f, $1e, $1d, $20, $0a
        .byte $0b, $0a, $21, $22, $0e, $0f, $10, $12
        .byte $0e, $0f, $10, $12, $0e, $23, $24, $be

        
        ; ---------- Channel 1 ----------
        .byte $25, $26, $27, $28, $29, $2a, $29, $2a
        .byte $29, $2a, $29, $2a, $2b, $2c, $2d, $2e
        .byte $2f, $30, $31, $32, $33, $34, $35, $36
        .byte $37, $34, $35, $36, $35, $38, $35, $39
        .byte $35, $38, $35, $39, $3a, $3a, $3b, $3c
        .byte $3b, $3d, $3c, $3a, $3c, $3d, $3e, $3f
        .byte $40, $41, $42, $43, $33, $34, $35, $36
        .byte $33, $34, $35, $36, $44, $45, $24, $fe


        .echo "Track size: ", *-tt_TrackDataStart   